/************************************************************************
	ds1302.c

    16x8 LED Display
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>
#include <stdio.h>
#include <string.h>

// Local includes
#include "hardware.h"
#include "ds1302.h"

void sendByteRTC(unsigned char byte)
{
	int bitNo;

	// Set IO to output
	RTC_IO_TRIS = WRITE;
	__delay_us(2);

	for (bitNo = 0; bitNo < 8; bitNo++)
	{
		// Place the bit on the pin
		if (byte & 0x01) RTC_IO = 1; else RTC_IO = 0;

		// Strobe the serial clock
		RTC_SCLK = 1;
		__delay_us(2);
		RTC_SCLK = 0;
		__delay_us(2);

		// Move the next bit into place
		byte >>= 1;
	}
}

unsigned char receiveByteRTC(void)
{
	unsigned char byte;
	int bitNo;

	byte = 0;

	// Set IO to input
	RTC_IO_TRIS = READ;
	__delay_us(2);

	for (bitNo = 0; bitNo < 8; bitNo++)
	{
		// Read the pin status and write the bit into the byte
		if (RTC_IO == 1) byte |= 0b00000001 << bitNo;

		// Strobe the serial clock
		RTC_SCLK = 1;
		__delay_us(2);
		RTC_SCLK = 0;
		__delay_us(2);
	}

	return byte;
}

void initRTC(void)
{
	// Chip enable high
	RTC_CE = ON;
	__delay_us(2);

	// Disable write protection
	sendByteRTC(WP + RTC_WRITE);
	sendByteRTC(0b00000000);

	// Set the clock halt flag to FALSE
	sendByteRTC(SECONDS + RTC_WRITE);
	sendByteRTC(0b00000000);

	// Chip enable low
	RTC_CE = OFF;
	__delay_us(2);
}

void setRTC(void)
{
	// Chip enable high
	RTC_CE = ON;
	__delay_us(2);

	// Request a clock write burst
	sendByteRTC(CLOCK_BURST + RTC_WRITE);

	// Write the seconds (converting decimal to BCD)
	sendByteRTC(((datetime.seconds/10)<<4)+(datetime.seconds%10));

	// Write the minutes (converting decimal to BCD)
	sendByteRTC(((datetime.minutes/10)<<4)+(datetime.minutes%10));

	// Write the hours (converting decimal to BCD)
	sendByteRTC(((datetime.hours/10)<<4)+(datetime.hours%10));

	// Write the day/date (converting decimal to BCD)
	sendByteRTC(((datetime.day/10)<<4)+(datetime.day%10));

	// Write the month (converting decimal to BCD)
	sendByteRTC(((datetime.month/10)<<4)+(datetime.month%10));

	// Write the day number (converting decimal to BCD)
	sendByteRTC(((datetime.dayNo/10)<<4)+(datetime.dayNo%10));

	// Write the year (converting decimal to BCD)
	sendByteRTC(((datetime.year/10)<<4)+(datetime.year%10));

	// Write the write protect (set to off)
	sendByteRTC(0b10000000);

	// Chip enable low
	RTC_CE = OFF;
	__delay_us(2);
}

void readRTC(void)
{
	unsigned char temp;
	unsigned char string[24];

	// Chip enable high
	RTC_CE = ON;
	__delay_us(2);

	// Request a clock burst (returns 8 bytes)
	sendByteRTC(CLOCK_BURST + RTC_READ);

	// Read the seconds (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.seconds = ((temp >> 4)*10+(temp & 0x0F));

	// Read the minutes (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.minutes = ((temp >> 4)*10+(temp & 0x0F));

	// Read the hours  (in BCD and convert to decimal - 24hr format only)
	temp = receiveByteRTC();
	datetime.hours = ((temp >> 4)*10+(temp & 0x0F));

	// Read the day/date (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.day = ((temp >> 4)*10+(temp & 0x0F));

	// Read the month (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.month = ((temp >> 4)*10+(temp & 0x0F));

	// Read the day of the week number (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.dayNo = ((temp >> 4)*10+(temp & 0x0F));

	// Read the year (in BCD and convert to decimal)
	temp = receiveByteRTC();
	datetime.year = ((temp >> 4)*10+(temp & 0x0F));

	// Read the write protect (and discard as we don't need it)
	temp = receiveByteRTC();

	// Chip Enable low
	RTC_CE = OFF;
	__delay_us(2);
}

void timeString(char *string)
{
	unsigned char number1, number2;

	// Assemble a time string
	if (datetime.hours < 10) number1 = 0;
	else number1 = datetime.hours / 10;

	number2 = datetime.hours - (number1 * 10);

	// Generate a null terminated ASCII string
	string[0] = number1 + 48;
	string[1] = number2 + 48;
	string[2] = ':';

	if (datetime.minutes < 10) number1 = 0;
	else number1 = datetime.minutes / 10;

	number2 = datetime.minutes - (number1 * 10);

	// Generate a null terminated ASCII string
	string[3] = number1 + 48;
	string[4] = number2 + 48;
	string[5] = '\0';
}

void dateString(char *string)
{
	unsigned char number1, number2;
	int length;

	// Blank the string
	string[0] = '\0';

	switch(datetime.dayNo)
	{
		case 1:	strcat(string, "Monday ");
				break;
		case 2:	strcat(string, "Tuesday ");
				break;
		case 3:	strcat(string, "Wednesday ");
				break;
		case 4:	strcat(string, "Thursday ");
				break;
		case 5:	strcat(string, "Friday ");
				break;
		case 6:	strcat(string, "Saturday ");
				break;
		case 7:	strcat(string, "Sunday ");
				break;
	}

	strcat(string, "the ");

	if (datetime.day < 10) number1 = 0;
	else number1 = datetime.day / 10;

	number2 = datetime.day - (number1 * 10);

	if (number1 > 0)
	{
		length = strlen(string);
		string[length] = number1 + 48;
		string[length+1] = '\0';
	}

	length = strlen(string);
	string[length] = number2 + 48;
	string[length+1] = '\0';

	if (number1 == 1)
	{
		strcat(string, "th of ");
	}
	else
	{
		if (number2 == 1) strcat(string, "st of ");
		else if (number2 == 2) strcat(string, "nd of ");
		else if (number2 == 3) strcat(string, "rd of ");
		else strcat(string, "th of ");
	}

	switch(datetime.month)
	{
		case 1:		strcat(string, "January ");
					break;
		case 2:		strcat(string, "Febuary ");
					break;
		case 3:		strcat(string, "March ");
					break;
		case 4:		strcat(string, "April ");
					break;
		case 5:		strcat(string, "May ");
					break;
		case 6:		strcat(string, "June ");
					break;
		case 7:		strcat(string, "July ");
					break;
		case 8:		strcat(string, "August ");
					break;
		case 9:		strcat(string, "September ");
					break;
		case 10:	strcat(string, "October ");
					break;
		case 11:	strcat(string, "November ");
					break;
		case 12:	strcat(string, "December ");
					break;
	}

	strcat(string, "20");

	if (datetime.year < 10) number1 = 0;
	else number1 = datetime.year / 10;

	number2 = datetime.year - (number1 * 10);

	length = strlen(string);
	string[length] = number1 + 48;
	string[length+1] = number2 + 48;
	string[length+2] = '\0';
}