/************************************************************************
	input.c

    16x8 LED Display
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "input.h"
#include "display.h"

// This routine gets a number between 00 and 99
// it implements physical keyswitch debounce and also
// two repeat timers, one for the first repeat and then
// a quicker subsiquent repeat counter.
unsigned char getNumber(unsigned char min, unsigned char max)
{
	unsigned char done = FALSE;

	unsigned char upButtonState = OFF;
	unsigned char downButtonState = OFF;
	unsigned char enterButtonState = OFF;

	unsigned char upButtonDBcount = 0;
	unsigned char downButtonDBcount = 0;
	unsigned char enterButtonDBcount = 0;

	long upButtonRepeat = 0;
	long downButtonRepeat = 0;

	long upButtonRepeat2 = 0;
	long downButtonRepeat2 = 0;

	unsigned char result = min;
	unsigned char currentNumber = min;

	displayNumber(result);

	while (!done)
	{
		// ------------------------------------------------------------------------------
		// Get the current button states (with on and off debouncing)

		// Debounce up ON
		if (UP_BUTTON == OFF && upButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (upButtonDBcount > 10)
				upButtonState = ON;
			else upButtonDBcount++;
		}

		// Debounce up OFF
		if (UP_BUTTON == ON && upButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (upButtonDBcount > 10)
				upButtonState = OFF;
			else upButtonDBcount++;
		}

		// Reset debounce counter if required
		if (UP_BUTTON != upButtonState) upButtonDBcount = 0;

		// Debounce down ON
		if (DOWN_BUTTON == OFF && downButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (downButtonDBcount > 5)
				downButtonState = ON;
			else downButtonDBcount++;
		}

		// Debounce down OFF
		if (DOWN_BUTTON == ON && downButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (downButtonDBcount > 5)
				downButtonState = OFF;
			else downButtonDBcount++;
		}

		// Reset debounce counter if required
		if (DOWN_BUTTON != downButtonState) downButtonDBcount = 0;

		// Debounce enter ON
		if (ENTER_BUTTON == OFF && enterButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (enterButtonDBcount > 5)
				enterButtonState = ON;
			else enterButtonDBcount++;
		}

		// Debounce enter OFF
		if (ENTER_BUTTON == ON && enterButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (enterButtonDBcount > 5)
				enterButtonState = OFF;
			else enterButtonDBcount++;
		}

		// Reset debounce counter if required
		if (ENTER_BUTTON != enterButtonState) enterButtonDBcount = 0;

		// ------------------------------------------------------------------------------
		// Adjust the result and display

		if (upButtonState == ON && result < max)
		{
			if (upButtonRepeat < 30000) upButtonRepeat++;

			if (upButtonRepeat == 30000 || upButtonRepeat == 1)
			{
				result++;
				upButtonRepeat++;
			}

			if (upButtonRepeat > 30000)
			{
				upButtonRepeat2++;
				
				if (upButtonRepeat2 > 2000)
				{
					result++;
					upButtonRepeat2 = 0;
				}
			}
		}
		else
		{
			upButtonRepeat = 0;
			upButtonRepeat2 = 0;
		}


		if (downButtonState == ON && result > min)
		{
			if (downButtonRepeat < 30000) downButtonRepeat++;

			if (downButtonRepeat == 30000 || downButtonRepeat == 1)
			{
				result--;
				downButtonRepeat++;
			}

			if (downButtonRepeat > 30000)
			{
				downButtonRepeat2++;
				
				if (downButtonRepeat2 > 2000)
				{
					result--;
					downButtonRepeat2 = 0;
				}
			}
		}
		else
		{
			downButtonRepeat = 0;
			downButtonRepeat2 = 0;
		}

		if (enterButtonState == ON) done = TRUE;

		// Display the current number if it has changed
		if (currentNumber != result)
		{
			displayNumber(result);
			currentNumber = result;
		}
	}

	return result;
}

unsigned char getDay(void)
{
	unsigned char min = 1;
	unsigned char max = 7;

	unsigned char done = FALSE;

	unsigned char upButtonState = OFF;
	unsigned char downButtonState = OFF;
	unsigned char enterButtonState = OFF;

	unsigned char upButtonDBcount = 0;
	unsigned char downButtonDBcount = 0;
	unsigned char enterButtonDBcount = 0;

	long upButtonRepeat = 0;
	long downButtonRepeat = 0;

	long upButtonRepeat2 = 0;
	long downButtonRepeat2 = 0;

	unsigned char result = min;
	unsigned char currentNumber = min;

	switch(result)
	{
		case 1:		displayString("Mo");
					break;
		case 2:		displayString("Tu");
					break;
		case 3:		displayString("We");
					break;
		case 4:		displayString("Th");
					break;
		case 5:		displayString("Fr");
					break;
		case 6:		displayString("Sa");
					break;
		case 7:		displayString("Su");
					break;
	}

	while (!done)
	{
		// ------------------------------------------------------------------------------
		// Get the current button states (with on and off debouncing)

		// Debounce up ON
		if (UP_BUTTON == OFF && upButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (upButtonDBcount > 10)
				upButtonState = ON;
			else upButtonDBcount++;
		}

		// Debounce up OFF
		if (UP_BUTTON == ON && upButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (upButtonDBcount > 10)
				upButtonState = OFF;
			else upButtonDBcount++;
		}

		// Reset debounce counter if required
		if (UP_BUTTON != upButtonState) upButtonDBcount = 0;

		// Debounce down ON
		if (DOWN_BUTTON == OFF && downButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (downButtonDBcount > 5)
				downButtonState = ON;
			else downButtonDBcount++;
		}

		// Debounce down OFF
		if (DOWN_BUTTON == ON && downButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (downButtonDBcount > 5)
				downButtonState = OFF;
			else downButtonDBcount++;
		}

		// Reset debounce counter if required
		if (DOWN_BUTTON != downButtonState) downButtonDBcount = 0;

		// Debounce enter ON
		if (ENTER_BUTTON == OFF && enterButtonState == OFF)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (enterButtonDBcount > 5)
				enterButtonState = ON;
			else enterButtonDBcount++;
		}

		// Debounce enter OFF
		if (ENTER_BUTTON == ON && enterButtonState == ON)
		{
			// If the debounce tolerance is met change state otherwise
			// increment the debounce counter
			if (enterButtonDBcount > 5)
				enterButtonState = OFF;
			else enterButtonDBcount++;
		}

		// Reset debounce counter if required
		if (ENTER_BUTTON != enterButtonState) enterButtonDBcount = 0;

		// ------------------------------------------------------------------------------
		// Adjust the result and display

		if (upButtonState == ON && result < max)
		{
			if (upButtonRepeat < 30000) upButtonRepeat++;

			if (upButtonRepeat == 30000 || upButtonRepeat == 1)
			{
				result++;
				upButtonRepeat++;
			}

			if (upButtonRepeat > 30000)
			{
				upButtonRepeat2++;
				
				if (upButtonRepeat2 > 2000)
				{
					result++;
					upButtonRepeat2 = 0;
				}
			}
		}
		else
		{
			upButtonRepeat = 0;
			upButtonRepeat2 = 0;
		}


		if (downButtonState == ON && result > min)
		{
			if (downButtonRepeat < 30000) downButtonRepeat++;

			if (downButtonRepeat == 30000 || downButtonRepeat == 1)
			{
				result--;
				downButtonRepeat++;
			}

			if (downButtonRepeat > 30000)
			{
				downButtonRepeat2++;
				
				if (downButtonRepeat2 > 2000)
				{
					result--;
					downButtonRepeat2 = 0;
				}
			}
		}
		else
		{
			downButtonRepeat = 0;
			downButtonRepeat2 = 0;
		}

		if (enterButtonState == ON) done = TRUE;

		// Display the current day if it has changed
		if (currentNumber != result)
		{
			switch(result)
			{
				case 1:		displayString("Mo");
							break;
				case 2:		displayString("Tu");
							break;
				case 3:		displayString("We");
							break;
				case 4:		displayString("Th");
							break;
				case 5:		displayString("Fr");
							break;
				case 6:		displayString("Sa");
							break;
				case 7:		displayString("Su");
							break;
			}
			currentNumber = result;
		}
	}

	return result;
}
