/************************************************************************
	spi.c

    16x8 LED Display
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "spi.h"

// Write 16 bits to the A6276 controller
void LEDwrite(unsigned char byte1, unsigned char byte2)
{
	// Reverse the bytes
	byte1 = ((byte1>>1)&0x55)|((byte1<<1)&0xAA);
	byte1 = ((byte1>>2)&0x33)|((byte1<<2)&0xCC);
	byte1 = (byte1>>4) | (byte1<<4);

	byte2 = ((byte2>>1)&0x55)|((byte2<<1)&0xAA);
	byte2 = ((byte2>>2)&0x33)|((byte2<<2)&0xCC);
	byte2 = (byte2>>4) | (byte2<<4);

	// Disable latching
	CS_LED = 1;

	// Write the low byte
	SSPIF = 0;
	SSPBUF = byte1;
	while(!SSPIF); // Wait for transmit to complete

	// Write the high byte
	SSPIF = 0;
	SSPBUF = byte2;
	while(!SSPIF); // Wait for transmit to complete

	// Enable latching
	CS_LED = 0;
}

// Write the row via the 74HCL164 chip
void rowWrite(unsigned char row)
{
	unsigned char byte;

	// Select the row and store it in byte
	byte = 1 << row;

	// Chip enable
	CS_164 = 1;

	// Output the byte
	SSPIF = 0;
	SSPBUF = byte;
	while(!SSPIF); // Wait for transmit to complete

	// Chip disable
	CS_164 = 0;
}